--
-- SlapDown
-- Version: SDR 1.0.100205
-- Author: Walkerbo
-- Email: walkerbo.saurfang@gmail.com
-- Date: Wednesday 24 January 2024
--

-- I have tried to name variables, functions, frames, and buttons to be descriptive

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         metrics, lists & variables          ===========================
-- ===================================================================================================
-- ===================================================================================================

local addonName = "SlapDown"
local versionNumber = "SDR 1.0.100205"

local joinString1, joinString2 = " caused this fail by ", " and they "

local ImageList = {
    sdBanner = "Interface\\AddOns\\SlapDown\\Files\\SlapBanner",
    sdButton = "Interface\\Addons\\SlapDown\\Files\\SlapButton",
    sdButtonH = "Interface\\Addons\\SlapDown\\Files\\SlapButtonHighlight"
}

local NumberList = {
    bannerBoxSize = 100,
    bannerScale = 0.3,
    checkBoxSideGap = 100,
    checkBoxTopGap = -80,
    failEditBoxHeight = 25,
    failEditBoxWidth = 500,
    failTopGap = -200,
    labelSideGap = 10,
    labelSizeX = 50,
    labelSizeY = 20,
    labelTopGap = 160,
    percentageTextBoxHeight = 25,
    percentageTextBoxWidth = 40,
    scaleBoxGap = 250,
    scrollButtonHeight = 20,
    scrollButtonNumber = 7,
    scrollFrameHeight = 150,
    slapDownMagicButtonSize = 60
}

local ColourList = {
    colourText = "|cff3ef1be"
}

local StringList = {
    checkBoxText = " Always Show Slap Down Button",
    failInsultLabelText = "Fail Insult",
    failInsultNoString = "No insult phrase",
    failNotInRaid = "Not in a group",
    failReasonLabelText = "Fail Reason",
    failReasonNoString = "No reason phrase",
    scaleBoxText = "% Slap Button Scale"
}

local FontList = {
    checkBoxFontObject = NumberFontNormalLargeYellow
}

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         global keybinding names          ===========================
-- ===================================================================================================
-- ===================================================================================================

BINDING_HEADER_SLAPDOWNTITLE = "- Slap Down -"
BINDING_NAME_SLAPDOWNTOGGLESLAPDOWNOPTIONS = "Toggle Slap Down Options"
BINDING_NAME_SLAPDOWNRUNSLAPDOWNFUNCTION = "Slap Down Function"
BINDING_NAME_SLAPDOWNRESTSLAPDOWNBUTTON = "Reset Slap Down Button"

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         functions          ===========================
-- ===================================================================================================
-- ===================================================================================================

local function initialiseLists()
    if not SlapDownReasonList then
        SlapDownReasonList = {}
        SlapDownInsultList = {}
        SlapDownSettings = {
            showSlapDownButton = true,
            floatingButtonScale = 1
        }
    end
end

local function updateFailReasonList()
    FauxScrollFrame_Update(
        FailReasonListScrollFrame,
        #SlapDownReasonList,
        NumberList.scrollButtonNumber,
        NumberList.scrollButtonHeight
    )
    for index = 1, NumberList.scrollButtonNumber do
        local offset = index + FauxScrollFrame_GetOffset(FailReasonListScrollFrame)
        local button = FailReasonListScrollFrame.buttons[index]
        button:SetNormalFontObject(FontList.checkBoxFontObject)
        button:SetHighlightFontObject(FontList.checkBoxFontObject)
        button.index = offset
        if offset <= #SlapDownReasonList then
            button:SetText(SlapDownReasonList[offset])
            button:Show()
        else
            button:Hide()
        end
    end
end

local function updateFailInsultList()
    FauxScrollFrame_Update(
        FailInsultListScrollFrame,
        #SlapDownInsultList,
        NumberList.scrollButtonNumber,
        NumberList.scrollButtonHeight
    )
    for index = 1, NumberList.scrollButtonNumber do
        offset = index + FauxScrollFrame_GetOffset(FailInsultListScrollFrame)
        button = FailInsultListScrollFrame.buttons[index]
        button:SetNormalFontObject(FontList.checkBoxFontObject)
        button:SetHighlightFontObject(FontList.checkBoxFontObject)
        button.index = offset
        if offset <= #SlapDownInsultList then
            button:SetText(SlapDownInsultList[offset])
            button:Show()
        else
            button:Hide()
        end
    end
end

local function updateSlapDownButtonPercentage(scaleNumber)
    if scaleNumber >= 301 then -- if the entered number is too high
        scaleNumber = 300
    elseif scaleNumber <= 49 then -- if the entered number is too low
        scaleNumber = 50
    end
    saveScaleNumber = scaleNumber / 100
    ScaleSlapDownButtonEditBox:SetText(scaleNumber)
    SlapDownMagicButton:SetScale(saveScaleNumber)
    SlapDownSettings.floatingButtonScale = saveScaleNumber
end

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         global functions for keybinds       ===========================
-- ===================================================================================================
-- ===================================================================================================

function SLAPDOWNTOGGLESLAPDOWNOPTIONS()
    if SettingsPanel:IsShown() then
        HideUIPanel(SettingsPanel)
    else
        InterfaceOptionsFrame_OpenToCategory("Slap Down")
    end
end

function SLAPDOWNRUNSLAPDOWNFUNCTION()
    if IsInRaid() or IsInGroup() then
        if #SlapDownReasonList > 0 and #SlapDownInsultList > 0 then
            math.random() -- sometimes random is not so random so lets kick start it to help
            randomReasonString = math.random(1, #SlapDownReasonList)
            reasonString = (SlapDownReasonList[randomReasonString])

            math.random()
            randomInsultString = math.random(1, #SlapDownInsultList)
            insultString = (SlapDownInsultList[randomInsultString])

            numMembers = GetNumGroupMembers()
            math.random()
            randomNumber = math.random(1, numMembers)
            playerName = GetRaidRosterInfo(randomNumber)

            messageString = (playerName .. joinString1 .. reasonString .. joinString2 .. insultString)

            SendChatMessage(
                messageString,
                IsInGroup(LE_PARTY_CATEGORY_INSTANCE) and "INSTANCE_CHAT" or IsInRaid() and "RAID" or "PARTY"
            )
        else
            print(StringList.failMissingPhraseString)
        end
    else
        print(StringList.failNotInRaid)
    end
end

function SLAPDOWNRESTSLAPDOWNBUTTON()
    SlapDownMagicButton:ClearAllPoints()
    SlapDownMagicButton:SetPoint("CENTER", 0, 200)
    SlapDownMagicButton:Show()
    SlapDownSettings.showSlapDownButton = true
    updateSlapDownButtonPercentage(100)
end

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         slash commands          ===========================
-- ===================================================================================================
-- ===================================================================================================

SLASH_SLAPDOWNTOGGLESLAPDOWNOPTIONS1 = "/SlapDownOptions"
function SlashCmdList.SLAPDOWNTOGGLESLAPDOWNOPTIONS(msg, editbox)
    SLAPDOWNTOGGLESLAPDOWNOPTIONS()
end

SLASH_SLAPDOWNRUNSLAPDOWNFUNCTION1 = "/SlapDown"
function SlashCmdList.SLAPDOWNRUNSLAPDOWNFUNCTION(msg, editbox)
    SLAPDOWNRUNSLAPDOWNFUNCTION()
end

SLASH_SLAPDOWNRESTSLAPDOWNBUTTON1 = "/SlapDownPos"
function SlashCmdList.SLAPDOWNRESTSLAPDOWNBUTTON(msg, editbox)
    SLAPDOWNRESTSLAPDOWNBUTTON()
end

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         frames          ===========================
-- ===================================================================================================
-- ===================================================================================================

local SlapDownInterfaceFrame = CreateFrame("Frame")
SlapDownInterfaceFrame.name = "Slap Down"
SlapDownInterfaceFrame:SetScript(
    "OnEvent",
    function(self, event, ...)
        if event == "PLAYER_LOGIN" then
            print(ColourList.colourText .. " *** " .. addonName .. " - " .. versionNumber .. " Loaded *** \n") -- Load Message
            initialiseLists()
            updateFailReasonList()
            updateFailInsultList()

            if SlapDownSettings.showSlapDownButton then
                ShowSlapDownButtonCheckBox:SetChecked(true)
                SlapDownMagicButton:Show()
            else
                ShowSlapDownButtonCheckBox:SetChecked(false)
                SlapDownMagicButton:Hide()
            end

            updateSlapDownButtonPercentage(SlapDownSettings.floatingButtonScale * 100) -- set the scale number in the scale box
            ScaleSlapDownButtonEditBox:SetCursorPosition(0) -- ensure the scale number displays
            SlapDownMagicButton:SetScale(SlapDownSettings.floatingButtonScale) -- set button to scale
        elseif event == "PLAYER_ENTERING_WORLD" or event == "GROUP_FORMED" or event == "GROUP_JOINED" then
            if UnitInAnyGroup("player") or IsInRaid() or IsInGroup() or IsInInstance() then
                SlapDownMagicButton:Show()
            end
        elseif event == "GROUP_LEFT" then
            if UnitInAnyGroup("player") or IsInRaid() or IsInGroup() or IsInInstance() then
                SlapDownMagicButton:Show()
            elseif SlapDownSettings.showSlapDownButton then
                SlapDownMagicButton:Show()
            else
                SlapDownMagicButton:Hide()
            end
        end
    end
)
SlapDownInterfaceFrame:RegisterEvent("PLAYER_LOGIN") -- fires when a player logs in or reloads
SlapDownInterfaceFrame:RegisterEvent("PLAYER_ENTERING_WORLD") -- fires when the player sees a loading screen
SlapDownInterfaceFrame:RegisterEvent("GROUP_FORMED") -- fires when the player starts a group
SlapDownInterfaceFrame:RegisterEvent("GROUP_JOINED") -- fires when the player joins a group
SlapDownInterfaceFrame:RegisterEvent("GROUP_LEFT") -- fires when the player leaves a group
InterfaceOptions_AddCategory(SlapDownInterfaceFrame)

local SlapDownBanner = CreateFrame("Frame", "SlapDownBanner", SlapDownInterfaceFrame)
SlapDownBanner:SetSize(NumberList.bannerBoxSize, NumberList.bannerBoxSize)
SlapDownBanner:SetScale(NumberList.bannerScale)
SlapDownBanner:SetPoint("TOP", 0, 0)
SlapDownBanner.Texture = SlapDownBanner:CreateTexture(nil, "BACKGROUND")
SlapDownBanner.Texture:SetPoint("TOP", 0, 0)
SlapDownBanner.Texture:SetTexture(ImageList.sdBanner)

local ShowSlapDownButtonCheckBox =
    CreateFrame("CheckButton", "ShowSlapDownButtonCheckBox", SlapDownInterfaceFrame, "UICheckButtonTemplate")
ShowSlapDownButtonCheckBox:SetPoint(
    "TOPRIGHT",
    SlapDownInterfaceFrame,
    "TOPLEFT",
    NumberList.checkBoxSideGap,
    NumberList.checkBoxTopGap
)
ShowSlapDownButtonCheckBox.text:SetFontObject(FontList.checkBoxFontObject)
ShowSlapDownButtonCheckBox.text:SetText(StringList.checkBoxText)
ShowSlapDownButtonCheckBox:SetScript(
    "OnClick",
    function()
        if SlapDownSettings.showSlapDownButton then
            SlapDownSettings.showSlapDownButton = false
            SlapDownMagicButton:Hide()
        else
            SlapDownSettings.showSlapDownButton = true
            SlapDownMagicButton:Show()
        end
    end
)

local ScaleSlapDownButtonEditBox =
    CreateFrame("EditBox", "ScaleSlapDownButtonEditBox", SlapDownInterfaceFrame, "ChatConfigBoxTemplate")
ScaleSlapDownButtonEditBox:SetPoint("LEFT", ShowSlapDownButtonCheckBox, "RIGHT", NumberList.scaleBoxGap, 0)
ScaleSlapDownButtonEditBox:SetFontObject(FontList.checkBoxFontObject)
ScaleSlapDownButtonEditBox:SetNumeric(true)
ScaleSlapDownButtonEditBox:SetSize(NumberList.percentageTextBoxWidth, NumberList.percentageTextBoxHeight)
ScaleSlapDownButtonEditBox:SetMaxLetters(3)
ScaleSlapDownButtonEditBox:SetAutoFocus()
ScaleSlapDownButtonEditBox:SetJustifyH("CENTER")
ScaleSlapDownButtonEditBox:SetScript(
    "OnEnterPressed",
    function(self)
        scaleNumber = tonumber(ScaleSlapDownButtonEditBox:GetText())
        self:ClearFocus()
        updateSlapDownButtonPercentage(scaleNumber)
    end
)

ScaleSlapDownButtonEditBox.Text = ScaleSlapDownButtonEditBox:CreateFontString("ScaleSlapDownButtonEditBoxText")
ScaleSlapDownButtonEditBoxText:SetPoint("LEFT", ScaleSlapDownButtonEditBox, "RIGHT")
ScaleSlapDownButtonEditBoxText:SetFontObject(FontList.checkBoxFontObject)
ScaleSlapDownButtonEditBoxText:SetText(StringList.scaleBoxText)

local FailReasonLabelBox = CreateFrame("Frame", "FailReasonLabelBox", SlapDownInterfaceFrame)
FailReasonLabelBox:SetPoint("LEFT", SlapDownInterfaceFrame, "LEFT", NumberList.labelSideGap, NumberList.labelTopGap)
FailReasonLabelBox:SetSize(NumberList.labelSizeX, NumberList.labelSizeY)
FailReasonLabelBox.Text = FailReasonLabelBox:CreateFontString("FailReasonLabelBoxText")
FailReasonLabelBoxText:SetPoint("LEFT", FailReasonLabelBox)
FailReasonLabelBoxText:SetFontObject(FontList.checkBoxFontObject)
FailReasonLabelBoxText:SetText(StringList.failReasonLabelText)

local EnterFailReasonEditBox =
    CreateFrame("EditBox", "EnterFailReasonEditBox", SlapDownInterfaceFrame, "ChatConfigBoxTemplate")
EnterFailReasonEditBox:SetPoint("TOPLEFT", FailReasonLabelBox, "BOTTOMLEFT")
EnterFailReasonEditBox:SetFontObject(FontList.checkBoxFontObject)
EnterFailReasonEditBox:SetAutoFocus()
EnterFailReasonEditBox:SetJustifyH("LEFT")
EnterFailReasonEditBox:SetSize(NumberList.failEditBoxWidth, NumberList.failEditBoxHeight)
EnterFailReasonEditBox:SetTextInsets(10, 10, 0, 0)
EnterFailReasonEditBox:SetScript(
    "OnEnterPressed",
    function(self)
        failReasonString = tostring(EnterFailReasonEditBox:GetText())
        if failReasonString ~= "" then
            table.insert(SlapDownReasonList, 1, failReasonString)
            self:ClearFocus()
            self:SetText("")
            updateFailReasonList()
        else
            print("empty string")
        end
    end
)

local FailListReasonScrollParent =
    CreateFrame("Frame", "FailListReasonScrollParent", SlapDownInterfaceFrame, "ChatConfigBoxTemplate")
FailListReasonScrollParent:SetSize(NumberList.failEditBoxWidth, NumberList.scrollFrameHeight)
FailListReasonScrollParent:SetPoint("TOPLEFT", EnterFailReasonEditBox, "BOTTOMLEFT")

local FailReasonListScrollFrame =
    CreateFrame("ScrollFrame", "FailReasonListScrollFrame", FailListReasonScrollParent, "FauxScrollFrameTemplate")
FailReasonListScrollFrame:SetPoint("TOPLEFT", 0, -8)
FailReasonListScrollFrame:SetPoint("BOTTOMRIGHT", -30, 8)
FailReasonListScrollFrame:SetScript(
    "OnVerticalScroll",
    function(self, offset)
        FauxScrollFrame_OnVerticalScroll(self, offset, NumberList.scrollButtonHeight, updateFailReasonList)
    end
)

FailReasonListScrollFrame.buttons = {}
for index = 1, NumberList.scrollButtonNumber do
    FailReasonListScrollFrame.buttons[index] =
        CreateFrame("Button", "$parentbtn" .. index, FailListReasonScrollParent, "OptionsListButtonTemplate")
    local button = FailReasonListScrollFrame.buttons[index]
    button:SetSize(NumberList.failEditBoxWidth, NumberList.scrollButtonHeight)
    button:SetPoint("TOPLEFT", 8, -(index - 1) * NumberList.scrollButtonHeight - 8)
    button:SetScript(
        "OnClick",
        function(self, button)
            if button == "RightButton" then
                table.remove(SlapDownReasonList, self.index)
                updateFailReasonList()
            end
        end
    )
end

local FailInsultLabelBox = CreateFrame("Frame", "FailInsultLabelBox", SlapDownInterfaceFrame)
FailInsultLabelBox:SetPoint("BOTTOMLEFT", FailListReasonScrollParent, "TOPLEFT", 0, NumberList.failTopGap)
FailInsultLabelBox:SetSize(NumberList.labelSizeX, NumberList.labelSizeY)
FailInsultLabelBox.Text = FailInsultLabelBox:CreateFontString("FailInsultLabelBoxText")
FailInsultLabelBoxText:SetPoint("LEFT", FailInsultLabelBox)
FailInsultLabelBoxText:SetFontObject(FontList.checkBoxFontObject)
FailInsultLabelBoxText:SetText(StringList.failInsultLabelText)

local EnterFailInsultEditBox =
    CreateFrame("EditBox", "EnterFailInsultEditBox", SlapDownInterfaceFrame, "ChatConfigBoxTemplate")
EnterFailInsultEditBox:SetPoint("TOPLEFT", FailInsultLabelBox, "BOTTOMLEFT")
EnterFailInsultEditBox:SetFontObject(FontList.checkBoxFontObject)
EnterFailInsultEditBox:SetAutoFocus()
EnterFailInsultEditBox:SetJustifyH("LEFT")
EnterFailInsultEditBox:SetSize(NumberList.failEditBoxWidth, NumberList.failEditBoxHeight)
EnterFailInsultEditBox:SetTextInsets(10, 10, 0, 0)
EnterFailInsultEditBox:SetScript(
    "OnEnterPressed",
    function(self)
        failInsultString = tostring(EnterFailInsultEditBox:GetText())
        if failInsultString ~= "" then
            table.insert(SlapDownInsultList, 1, failInsultString)
            self:ClearFocus()
            self:SetText("")
            updateFailInsultList()
        else
            print("empty string")
        end
    end
)

local FailListInsultScrollParent =
    CreateFrame("Frame", "FailListInsultScrollParent", SlapDownInterfaceFrame, "ChatConfigBoxTemplate")
FailListInsultScrollParent:SetSize(NumberList.failEditBoxWidth, NumberList.scrollFrameHeight)
FailListInsultScrollParent:SetPoint("TOPLEFT", EnterFailInsultEditBox, "BOTTOMLEFT")

local FailInsultListScrollFrame =
    CreateFrame("ScrollFrame", "FailInsultListScrollFrame", FailListInsultScrollParent, "FauxScrollFrameTemplate")
FailInsultListScrollFrame:SetPoint("TOPLEFT", 0, -8)
FailInsultListScrollFrame:SetPoint("BOTTOMRIGHT", -30, 8)
FailInsultListScrollFrame:SetScript(
    "OnVerticalScroll",
    function(self, offset)
        FauxScrollFrame_OnVerticalScroll(self, offset, NumberList.scrollButtonHeight, updateFailInsultList)
    end
)

FailInsultListScrollFrame.buttons = {}
for index = 1, NumberList.scrollButtonNumber do
    FailInsultListScrollFrame.buttons[index] =
        CreateFrame("Button", "$parentbtn" .. index, FailListInsultScrollParent, "OptionsListButtonTemplate")
    local button = FailInsultListScrollFrame.buttons[index]
    button:SetSize(NumberList.failEditBoxWidth, NumberList.scrollButtonHeight)
    button:SetPoint("TOPLEFT", 8, -(index - 1) * NumberList.scrollButtonHeight - 8)
    button:SetScript(
        "OnClick",
        function(self, button)
            if button == "RightButton" then
                table.remove(SlapDownInsultList, self.index)
                updateFailInsultList()
            end
        end
    )
end

local SlapDownMagicButton = CreateFrame("Button", "SlapDownMagicButton", UIParent)
SlapDownMagicButton:SetSize(NumberList.slapDownMagicButtonSize, NumberList.slapDownMagicButtonSize)
SlapDownMagicButton:ClearAllPoints()
SlapDownMagicButton:SetPoint("CENTER", 0, 200)
SlapDownMagicButton:SetNormalTexture(ImageList.sdButton)
SlapDownMagicButton:SetHighlightTexture(ImageList.sdButtonH, "BLEND")
SlapDownMagicButton:SetMovable(true)
SlapDownMagicButton:SetClampedToScreen(true)
SlapDownMagicButton:RegisterForDrag("MiddleButton")
SlapDownMagicButton:SetScript("OnDragStart", SlapDownMagicButton.StartMoving)
SlapDownMagicButton:SetScript("OnDragStop", SlapDownMagicButton.StopMovingOrSizing)
SlapDownMagicButton:RegisterForClicks("RightButtonUp", "LeftButtonUp")
SlapDownMagicButton:SetToplevel(true)
SlapDownMagicButton:SetScript(
    "OnClick",
    function(self, button)
        if button == "LeftButton" then
            SLAPDOWNRUNSLAPDOWNFUNCTION()
        elseif button == "RightButton" then
            SLAPDOWNTOGGLESLAPDOWNOPTIONS()
        end
    end
)
